
dcomplex InverseMellinIntegrand(dcomplex N, double log1ox, dcomplex f(dcomplex, void*), void *par) {
  return exp( N*log1ox ) * f(N,par) / M_PI;
}
double InverseMellin(double z, dcomplex f(dcomplex, void*), void *par) { // Talbot algorithm
  if(z==1.) return 0;
  double log1ox = -log(z);
  double res = 0.; 
  // prec ~ 10^{-0.6*M}
  int M = 13;
  double r = 1.+2.*M/5./log1ox;
  double t, cott, sigma;
  dcomplex N;
  res = real(0.5 * InverseMellinIntegrand(r, log1ox, f, par));
  for(int k=1; k<M; k++) {
    t = k*M_PI/M;
    cott = 1./tan(t);
    sigma = t + cott*(t*cott - 1.);
    N = r*t*(cott+I);
    res += real(InverseMellinIntegrand(N, log1ox, f, par) * (1.+I*sigma));
  }
  res *= M_PI*r/M;
  return res;
}







const double A1 = CA*CA*(67./36.-ZETA2/2.) - 5./18.*CA*_nf;
const double A2 = CA*CA*CA*(245./24.-67./9.*ZETA2+11./6.*ZETA3+11./5.*ZETA2*ZETA2)/4.
  + CA*CA*_nf*(-209./108.+10./9.*ZETA2-7./3.*ZETA3)/4.
  + CA*CF*_nf*(-55./24.+2.*ZETA3)/4.
  - CA*_nf*_nf/4./27.;
const double B1 = CA*CA*(2./3.+3./4.*ZETA3) - (CA/6.+CF/8.)*_nf;
const double B2 = CA*CA*CA/4.*(79./32.-ZETA2*ZETA3+ZETA2/6.+11./24.*ZETA2*ZETA2+67./6.*ZETA3-5.*ZETA5)
  - CA*CA*_nf/4.*(233./288.+ZETA2/6.+ZETA2*ZETA2/12.+5./3.*ZETA3)
  - CA*CF*_nf/4.*241./288.
  + CF*CF*_nf/4./16.
  + _nf*_nf/4.*(CA*29./288.+CF*11./144.);
//
// resummation coeffs
const double rA1 = CA/M_PI;
const double rA2 = CA/2.* (67.*CA/18.-5./9.*_nf-CA*ZETA2) /M_PI/M_PI;
const double rA3 = ( CA*CA* (245./24.-67.*ZETA2/9.+11.*ZETA3/6.+11.*ZETA2*ZETA2/5.) +
		     ( (2.*ZETA3 - 55./24.)*CF + (10.*ZETA2/9.-7.*ZETA3/3.-209./108.)*CA )*_nf
		     - _nf*_nf*1./27. ) *CA/4. /M_PI/M_PI/M_PI;
const double rD2 = CA * ( CA * (176.*ZETA2/3. + 56.*ZETA3 - 1616./27.) + _nf*(224./27.-32.*ZETA2/3.) )/16./M_PI/M_PI; // checked: OK.
const double rD3 = CA * ( CA*CA *(-594058./729. +98224./81.*ZETA2 +40144./27.*ZETA3 -2992./15.*ZETA2*ZETA2 -352./3.*ZETA2*ZETA3 -384.*ZETA5 )
			  +CA*_nf*( 125252./729. -29392./81.*ZETA2 -2480./9.*ZETA3 +736./15.*ZETA2*ZETA2 )
			  +CF*_nf*( 3422./27. - 32.*ZETA2 - 608./9.*ZETA3 - 64./5.*ZETA2*ZETA2 )
			  +_nf*_nf*( 320./27.*ZETA3 + 640./27.*ZETA2 - 3712./729.)) /64./M_PI/M_PI/M_PI; // checked: OK.
//
double g0bar1=0, g0bar2=0, g0bar3=0;
void initResConsts(double M) {
  //
  double W1 = (finite_mt ? _W1t : _W1);
  double W2 = (finite_mt ? _W2t : _W2);
  double W3 = (finite_mt ? _W3t : _W3);
  double d1 = _cd1 + W1;
  double d2 = _cd2 + _cd1*W1 + W2;
  double d3 = _cd3 + _cd2*W1 + _cd1*W2 + W3;
  //
  double gA1 = CA*CA*(67./36.-ZETA2/2.) - 5./18.*CA*_nf;
  double gB1 = CA*CA*(2./3.+3./4.*ZETA3) - (CA/6.+CF/8.)*_nf;
  double gB2 = CA*CA*CA/4.*(79./32.-ZETA2*ZETA3+ZETA2/6.+11./24.*ZETA2*ZETA2+67./6.*ZETA3-5.*ZETA5)
    - CA*CA*_nf/4.*(233./288.+ZETA2/6.+ZETA2*ZETA2/12.+5./3.*ZETA3)
    - CA*CF*_nf/4.*241./288.
    + CF*CF*_nf/4./16.
    + _nf*_nf/4.*(CA*29./288.+CF*11./144.);
  //
  double lf  = 2.*log(M/muF);
  double lrf = 2.*log(muR/muF);
  double lr = 2.*log(muR/M);
  //
  //double a11 = 4*CA;
  double a23 = 8*CA*CA;
  double a22 = -4*CA*b0;
  double a21 = 4*CA*d1 + CA*CA*(67./9.-18.*ZETA2) - 10./9.*CA*_nf;
  double a20 = CA*CA*(-101./27.+11./3.*ZETA2+39./2.*ZETA3) + CA*_nf*(14./27.-2./3.*ZETA2);
  // factorization scale dependence (as=as(M))
  d3 += (4*b0*b0*b0 - 6*CA*CA*b0*ZETA2 + 8./3.*CA*CA*CA*ZETA3)*lf*lf*lf
    //
    + (b0*(6*gB1+b1) - 4*CA*gA1*ZETA2 + 20*CA*CA*b0*ZETA3 + 8*CA*CA*CA*(ZETA2*ZETA2-3*ZETA4) + d1*(3*b0*b0-2*CA*CA*ZETA2))*lf*lf
    //
    + (2*gB2+8*CA*gA1*ZETA3+2*gB1*d1+2*b0*d2+2*CA*(6*ZETA5*a23-2*ZETA4*a22+ZETA3*a21-ZETA2*a20))*lf;
  d2 += (-2.*CA*CA*ZETA2 + 3*b0*b0)*lf*lf + (2*gB1+2*b0*d1+8*CA*CA*ZETA3)*lf;
  d1 += 2*b0*lf;
  //
  // adding renormalization scale dependence
  d3 += (2*b2+3*b1*d1+4*b0*d2)*lr + (7*b0*b1+6*b0*b0*d1)*lr*lr + 4*pow(b0*lr,3);
  d2 += (2*b1+3*b0*d1)*lr + 3*b0*b0*lr*lr;
  d1 += 2*b0*lr;
  //
  // restore normalization
  d1 /= M_PI;
  d2 /= M_PI*M_PI;
  d3 /= M_PI*M_PI*M_PI;
  //
  // construct full g0k
  //const double c1 = EulerGamma;
  double bb0 = b0/M_PI;
  double bb1 = b1/b0/M_PI;
  double c2 = pow(EulerGamma,2) -ZETA2;
  double c3 = pow(EulerGamma,3) -3*EulerGamma*ZETA2 +2*ZETA3;
  double c4 = pow(EulerGamma,4) -6*EulerGamma*EulerGamma*ZETA2 +8*EulerGamma*ZETA3 +3*ZETA2*ZETA2 -6*ZETA4;
  double c5 = 24*ZETA5 +3*EulerGamma*ZETA2*ZETA2 +pow(EulerGamma,5) - 10*pow(EulerGamma,3)*ZETA2 +20*EulerGamma*EulerGamma*ZETA3 -20*ZETA3*ZETA2;
  double c6 = 144*ZETA5*EulerGamma +pow(EulerGamma,6) -15*pow(EulerGamma,4)*ZETA2 +40*pow(EulerGamma,3)*ZETA3 +9*EulerGamma*EulerGamma*ZETA2*ZETA2 -120*EulerGamma*ZETA3*ZETA2 +40*ZETA3*ZETA3 -45./7.*pow(ZETA2,3);
  //
  double S1 = 2*rA1* (ZETA2+EulerGamma*EulerGamma - EulerGamma*lf);
  double S2 = 4*rA1*bb0/3.*c3 -2*( rA1*bb0*(2*EulerGamma+lr) + rA2 )*c2
    - EulerGamma* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*bb0*(EulerGamma*EulerGamma+ZETA2)
		    - 4.*rA1*bb0*EulerGamma*lr +2*rA2*lf
		    - rA1*bb0*(lr*lr - lrf*lrf) );
  double S3 = -c4* (4*rA1*pow(bb0,2))/3.
    +c3* (4*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma) - 4*rA1*bb0*log(M/muR)))/3.
    -c2* 2*(rA3 + bb0*(-rD2 + 4*rA2*EulerGamma + 
		     2*rA1*(bb1*EulerGamma + 2*bb0*(pow(EulerGamma,2) + ZETA2))) - 
	    2*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma))*log(M/muR) + 
	    4*rA1*pow(bb0,2)*pow(log(M/muR),2))
    -EulerGamma* (3*rD3 - 12*rA3*EulerGamma + 12*bb0*rD2*EulerGamma - 24*rA2*bb0*pow(EulerGamma,2) - 
		  12*rA1*bb0*bb1*pow(EulerGamma,2) - 16*rA1*pow(bb0,2)*pow(EulerGamma,3) - 
		  24*rA2*bb0*ZETA2 - 12*rA1*bb0*bb1*ZETA2 - 48*rA1*pow(bb0,2)*EulerGamma*ZETA2 - 
		  32*rA1*pow(bb0,2)*ZETA3 + 12*
		  (rA3 + bb0*(-rD2 + 4*rA2*EulerGamma + 
			    2*rA1*(bb1*EulerGamma + 2*bb0*(pow(EulerGamma,2) + ZETA2))))*log(M/muR) - 
		  12*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma))*pow(log(M/muR),2) + 
		  16*rA1*pow(bb0,2)*pow(log(M/muR),3) - 12*rA3*log(muF/muR) + 
		  12*bb0*(2*rA2 + rA1*bb1)*pow(log(muF/muR),2) - 
		  16*rA1*pow(bb0,2)*pow(log(muF/muR),3))/3.;
  // S1^2
  const double S1s = -4*rA1*rA1*(c4 -2*(2*EulerGamma-lf)*c3 +pow(2*EulerGamma-lf,2)*c2);
  // S1^3
  const double S1c = -8*rA1*rA1*rA1*(c6 -3*(2*EulerGamma-lf)*c5 +3*pow(2*EulerGamma-lf,2)*c4 -pow(2*EulerGamma-lf,3)*c3);
  // S1*S2
  const double S1S2 = 2*rA1*(4*rA1*bb0/3.*c5 -2*( rA1*bb0*(2*EulerGamma+lr) + rA2 )*c4
			    - c3* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*bb0*(EulerGamma*EulerGamma+ZETA2)
				    - 4.*rA1*bb0*EulerGamma*lr +2*rA2*lf
				    - rA1*bb0*(lr*lr - lrf*lrf) ))
    -2*rA1*(2*EulerGamma-lf)* (4*rA1*bb0/3.*c4 -2*( rA1*bb0*(2*EulerGamma+lr) + rA2 )*c3
			      - c2* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*bb0*(EulerGamma*EulerGamma+ZETA2)
				      - 4.*rA1*bb0*EulerGamma*lr +2*rA2*lf
				      - rA1*bb0*(lr*lr - lrf*lrf) ));
  //
  const double gD1 = S1;
  const double gD2 = S1s/2. + S2 + gD1*gD1;
  const double gD3 = S1c/6. + S1S2 + S3 + 2*gD1*gD2 - gD1*gD1*gD1;
  //
  double g01 = d1 + gD1;
  double g02 = d2 + d1*gD1 + gD2;
  double g03 = d3 + d2*gD1 + d1*gD2 + gD3; 
  //
  g0bar1 = d1;
  g0bar2 = g02 - g01*S1 - S2 + S1*S1/2.;
  g0bar3 = g03 - g02*S1 - g01*(S2-S1*S1/2.) - (S1*S1*S1/6. - S1*S2 + S3);
  //
  return;
}
//


// Pointlike log contributions
//
dcomplex mellD(int k, dcomplex N) {
  if(__log_type__ == 0)
    return mell_D(k,N);
  else if(__log_type__ == 1)
    return mell_DB(k,N);
  else if(__log_type__ == 2)
    return mell_DM(k,N);
  else if(__log_type__ == 3)
    return mell_DMconst(k,N);
  return 0;
}
//
dcomplex S1(dcomplex N) {
  return 4.*CA*mellD(1,N);
}
dcomplex S2(dcomplex N) {
  dcomplex res = ( (2./3.*_nf - 11./3.*CA)*CA * mellD(2,N)
		  + (CA*CA*(67./9. - 2.*ZETA2) - CA*10./9.*_nf) * mellD(1,N)
		  + (CA*CA*(-101./27. + 11./3.*ZETA2 + 7./2.*ZETA3) + _nf*CA*(14./27.-2./3.*ZETA2)) * mellD(0,N)
		  );
  return res;
}
dcomplex S1_sq(dcomplex N) {
  return 16*CA*CA*(mellD(3,N) - 2.*ZETA2*mellD(1,N) + 2.*ZETA3*mellD(0,N) - ZETA2*ZETA2/10.);
}
dcomplex S1_cube(dcomplex N) {
  return 64*CA*CA*CA*( 3./4.*mellD(5,N) - 6.*ZETA2*mellD(3,N) + 15.*ZETA3*mellD(2,N) -3./2.*ZETA2*ZETA2*mellD(1,N)
		       +(18.*ZETA5-12.*ZETA2*ZETA3)*mellD(0,N) + 5*ZETA3*ZETA3 -29./35.*ZETA2*ZETA2*ZETA2);
}
dcomplex S1_S2(dcomplex N) {
  double b22 = (2./3.*_nf - 11./3.*CA)*CA;
  double b21 = CA*CA*(67./9. - 2.*ZETA2) - CA*10./9.*_nf;
  double b20 = CA*CA*(-101./27. + 11./3.*ZETA2 + 7./2.*ZETA3) + _nf*CA*(14./27.-2./3.*ZETA2);
  return 4*CA*( 5./6.*b22*mellD(4,N) + b21*mellD(3,N)
		+ (3./2.*b20-3.*ZETA2*b22) *mellD(2,N)
		+ (6.*ZETA3*b22-2.*ZETA2*b21) *mellD(1,N)
		+ (-ZETA2*ZETA2*b22+2.*ZETA3*b21-ZETA2*b20) *mellD(0,N)
		+ (4.*ZETA5-2.*ZETA2*ZETA3)*b22 -1./10.*ZETA2*ZETA2*b21 +ZETA3*b20
		);
}
//
dcomplex FO_2_log_PL(dcomplex N) {
  return C2log_pointlike_gg(N, mellD, _nf);
}
dcomplex FO_3_log_PL(dcomplex N) {
  return C3log_pointlike_gg(N, mellD, _nf);
}
//
dcomplex APimprovement(dcomplex f(dcomplex), dcomplex N) {
  dcomplex res = 0;
  if(APtype== 0) res = f(N);                                           // Ag(1)
  if(APtype== 1) res = f(N+1);                                         // Ag expanded to first order
  if(APtype== 2) res = 2*f(N  ) -3*f(N+1) +2*f(N+2);                   // full Ag expanded to second order
  if(APtype==-1) res =   f(N-1) -2*f(N  ) +3*f(N+1) -2*f(N+2) +f(N+3); // full Ag
  if(APtype==-2) res =(3*f(N+1) -2*f(N+2) +  f(N+3))/2.;               // soft Ags
  if(APtype==-3) res =   f(N-1);                                       // small-x
  return res;
}




// Expansion from resummation (with soft scale dependence)
int LL_order = 3;
dcomplex asS1(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex L1 = mellD(0,N);
  dcomplex L2 = 2*mellD(1,N) + 2*EulerGamma*L1;
  dcomplex value = as*2*L2*rA1;
  if(LL_order > 0) value += as*L1*4*(log(muM/muF)-EulerGamma)*rA1;
  return value;
}
dcomplex as2S2(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  double bb0 = b0/M_PI;
  dcomplex L1 = mellD(0,N);
  dcomplex L2 = 2*mellD(1,N) + 2*EulerGamma*L1;
  dcomplex L3 = 3*mellD(2,N)-3*ZETA2*L1 + 3*EulerGamma*L2 - 3*EulerGamma*EulerGamma*L1;
  dcomplex value = - as*as*4*L3*rA1*bb0/3.;
  if(LL_order > 0) value += as*as*L2* 2*( rA1*2*(EulerGamma-log(muM/muR))*bb0 + rA2 );
  if(LL_order > 1) value += as*as*L1* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*(EulerGamma*EulerGamma+ZETA2)*bb0
					+ 8.*rA1*EulerGamma*log(muM/muR)*bb0 +4.*rA2*log(muM/muF)
					- 4.*rA1*(pow(log(muM/muR),2.)-pow(log(muF/muR),2.))*bb0 );
  return value;
}
dcomplex as3S3(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  double bb0 = b0/M_PI;
  double bb1 = b1/b0/M_PI;
  dcomplex L1 = mellD(0,N);
  dcomplex L2 = 2*mellD(1,N) + 2*EulerGamma*L1;
  dcomplex L3 = 3*mellD(2,N)-3*ZETA2*L1 + 3*EulerGamma*L2 - 3*EulerGamma*EulerGamma*L1;
  dcomplex L4 = 4*mellD(3,N)-12*ZETA2*mellD(1,N)+8*ZETA3*L1 + 4*EulerGamma*L3 - 6*EulerGamma*EulerGamma*L2 + 4*EulerGamma*EulerGamma*EulerGamma*L1;
  dcomplex value = as*as*as*L4*4*rA1*bb0*bb0/3.;
  if(LL_order > 0) value += -as*as*as*L3*bb0*4./3.* (2*rA2 + rA1*(bb1 + 4*bb0*(EulerGamma-log(muM/muR))) );
  if(LL_order > 1) value += as*as*as*L2*2* ( rA3 + bb0*(-rD2 + 4*rA2*EulerGamma
						      + 2.*rA1*(bb1*EulerGamma + 2*bb0*(EulerGamma*EulerGamma+ZETA2)))
					     - 2*bb0*(2*rA2 + rA1*(bb1+4*bb0*EulerGamma))*log(muM/muR)
					     + 4*rA1*bb0*bb0*pow(log(muM/muR),2.) );
  if(LL_order > 2) value += as*as*as*L1/3.* (3*rD3 - 12*rA3*EulerGamma + 12*bb0*rD2*EulerGamma - 24*rA2*bb0*pow(EulerGamma,2)
					     - 12*rA1*bb0*bb1*pow(EulerGamma,2) - 16*rA1*pow(bb0,2)*pow(EulerGamma,3) - 24*rA2*bb0*ZETA2
					     - 12*rA1*bb0*bb1*ZETA2 - 48*rA1*pow(bb0,2)*EulerGamma*ZETA2 - 32*rA1*pow(bb0,2)*ZETA3
					     + 12*(rA3 + bb0*(-rD2 + 4*rA2*EulerGamma + 2*rA1*(bb1*EulerGamma + 2*bb0*(pow(EulerGamma,2) + ZETA2))))*log(muM/muR)
					     - 12*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma))*pow(log(muM/muR),2)
					     + 16*rA1*pow(bb0,2)*pow(log(muM/muR),3) - 12*rA3*log(muF/muR)
					     + 12*bb0*(2*rA2 + rA1*bb1)*pow(log(muF/muR),2) - 16*rA1*pow(bb0,2)*pow(log(muF/muR),3));
  return value;
}
//
dcomplex g0expS_order1(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex value;
  value = APimprovement(asS1,N);
  if(LL_order > 0) value += g0bar1*as;
  return value * M_PI;
}
dcomplex g0expS_order2(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex value;
  value = pow(APimprovement(asS1,N),2.)/2. + APimprovement(as2S2,N);
  if(LL_order > 0) value += g0bar1*as*APimprovement(asS1,N);
  if(LL_order > 1) value += g0bar2*as*as;
  return value * M_PI*M_PI;
}
dcomplex g0expS_order3(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex value;
  value = pow(APimprovement(asS1,N),3.)/6. + APimprovement(asS1,N)*APimprovement(as2S2,N) + APimprovement(as3S3,N);
  if(LL_order > 0) value += g0bar1*as*( pow(APimprovement(asS1,N),2.)/2. + APimprovement(as2S2,N) );
  if(LL_order > 1) value += g0bar2*as*as*APimprovement(asS1,N);
  if(LL_order > 2) value += g0bar3*as*as*as;
  return value * M_PI*M_PI*M_PI;
}
//
dcomplex lGamma(dcomplex z){
  gsl_sf_result r, arg;
  gsl_sf_lngamma_complex_e(real(z),imag(z),&r,&arg);  // causes problems in (x,0), x<0
  return polar(exp(r.val),arg.val);
}
bool dFMMV = false;
bool NextToSoft = false;
bool fullNS = false;
int maxorder = 8;
int maxLog = 3;
int NTSexpansion = 2;
dcomplex g0expS(dcomplex N) {
  dcomplex res = 0;
  if(order == 1) res = g0expS_order1(N);
  if(order == 2) res = g0expS_order2(N);
  if(order == 3) res = g0expS_order3(N);
  if(NextToSoft) {
    double W1 = (finite_mt ? _W1t : _W1);
    double W2 = (finite_mt ? _W2t : _W2);
    dcomplex L, dc1, dc2, dc3;
    if(__log_type__==1 && APtype>0) {
      L = psi0(N+1);
      dc1 = 0;
      dc2 = (31.4967*L*L +36.8633*L -6.85889)/N;
      dc3 = (189.*L*L*L*L + 546.708*L*L*L + 1197.47*L*L + 782.976*L -245.534)/N;
      //
      // for HXSWG
      if(APtype==2 && !fullNS) {
	dc1 = 0; // reset
	dc2 = 0; // reset
	dc3 = 0; // reset
	if(NTSexpansion == 0) {   // N-space with psi0(N)
	  L = psi0(N);
	  double c3k5[] = {0, 0, 0, -1296., 2592., -6480., 12960., -27216., 54432., -110160.};
	  double c3k4[] = {189., -61.0586, 754.059, -4238.27, 5803.68, -2831.34, -4045.14, 41105.7, -124248., 356251.};
	  double c3k3[] = {546.708, 156.626, 63.8296, -10150.5, 6991.69, -16682.6, 21726.9, -57884.1, 82044.2, -196390.};
	  double c3k2[] = {1197.47};
	  double c3k1[] = {782.976};
	  double c3k0[] = {-245.534};
	  for(int i=0; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += L*L*L*L*L*c3k5[i] /pow(N,i+1);
	    if(maxLog > 0) dc3 += L*L*L*L  *c3k4[i] /pow(N,i+1);
	    if(maxLog > 1) dc3 += L*L*L    *c3k3[i] /pow(N,i+1);
	    if(maxLog > 2) dc3 += L*L      *c3k2[i] /pow(N,i+1);
	    if(maxLog > 3) dc3 += L        *c3k1[i] /pow(N,i+1);
	    if(maxLog > 4) dc3 +=           c3k0[i] /pow(N,i+1);
	  }
	}
	if(NTSexpansion == 1) {   // N-space with psi0(N+1)
	  L = psi0(N+1);
	  double c3k5[] = {0, 0, 0, -1296., 2592., -6480., 12960., -27216., 54432., -110160.};
	  double c3k4[] = {189., -61.0586, 754.059, -4238.27, 12283.7, -15791.3, 28354.9, -23694.3, 11832.3, 84091.};
	  double c3k3[] = {546.708, -599.374, 308.064, -13166.7, 23944.8, -52857.3, 58972.3, -106504., 47221.5, 28441.3};
	  double c3k2[] = {1197.47};
	  double c3k1[] = {782.976};
	  double c3k0[] = {-245.534};
	  for(int i=0; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += L*L*L*L*L*c3k5[i] /pow(N,i+1);
	    if(maxLog > 0) dc3 += L*L*L*L  *c3k4[i] /pow(N,i+1);
	    if(maxLog > 1) dc3 += L*L*L    *c3k3[i] /pow(N,i+1);
	    if(maxLog > 2) dc3 += L*L      *c3k2[i] /pow(N,i+1);
	    if(maxLog > 3) dc3 += L        *c3k1[i] /pow(N,i+1);
	    if(maxLog > 4) dc3 +=           c3k0[i] /pow(N,i+1);
	  }
	}
	if(NTSexpansion == 2) {   // N-space with psi0(N+1) and 1/N/(N+1)/(N+2)...
	  L = psi0(N+1);
	  double c3k5[] = {0, 0, 0, -1296., -5184., -25920., -155520., -1.08864e6, -8.70912e6, -7.83821e7};
	  double c3k4[] = {189., -61.0586, 693., -2098.21, -5217.64, -5056.42, 81272.1, 1.24003e6, 1.45652e7, 1.67715e8};
	  double c3k3[] = {546.708, -599.374, -291.31, -13441.3, -55263.2, -273227., -1.64861e6, -1.17896e7, -9.73648e7, -9.10135e8};
	  double c3k2[] = {1197.47};
	  double c3k1[] = {782.976};
	  double c3k0[] = {-245.534};
	  for(int i=0; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += L*L*L*L*L*c3k5[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 0) dc3 += L*L*L*L  *c3k4[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 1) dc3 += L*L*L    *c3k3[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 2) dc3 += L*L      *c3k2[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 3) dc3 += L        *c3k1[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 4) dc3 +=           c3k0[i] *lGamma(N)/lGamma(N+1+i);
	  }
	}
	if(NTSexpansion == 3) {   // z-space
	  double a3k5[] = {0, 0, 0, 216., 216., 216., 216., 216., 216., 216.};
	  double a3k4[] = {189., -61.0586, 346.5, -1706.31, -1034.01, -696.744, -469.729, -297.999, -159.653, -43.714};
	  double a3k3[] = {-110.333, 702.633, -1255.44, 7675.63, 2971.25, 1516., 830.977, 454.002, 232.367, 100.178};
	  double a3k2[] = {-1236.77};
	  double a3k1[] = {2560.34};
	  double a3k0[] = {-1375.07};
	  for(int i=0; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += a3k5[i] *mell_L(i,5,N);
	    if(maxLog > 0) dc3 += a3k4[i] *mell_L(i,4,N);
	    if(maxLog > 1) dc3 += a3k3[i] *mell_L(i,3,N);
	    if(maxLog > 2) dc3 += a3k2[i] *mell_L(i,2,N);
	    if(maxLog > 3) dc3 += a3k1[i] *mell_L(i,1,N);
	    if(maxLog > 4) dc3 += a3k0[i] *mell_L(i,0,N);
	  }
	}
	if(NTSexpansion == 4) {   // z-space, divided by z
	  double a3k5[] = {0, 0, 0, 216., 0, 0, 0, 0, 0, 0};
	  double a3k4[] = {189., -250.059, 407.559, -2052.81, 672.3, 337.265, 227.015, 171.73, 138.346, 115.939};
	  double a3k3[] = {-110.333, 812.966, -1958.07, 8931.08, -4704.38, -1455.25, -685.021, -376.975, -221.635, -132.189};
	  double a3k2[] = {-1236.77};
	  double a3k1[] = {2560.34};
	  double a3k0[] = {-1375.07};
	  for(int i=0; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += a3k5[i] *mell_L(i,5,N-1);
	    if(maxLog > 0) dc3 += a3k4[i] *mell_L(i,4,N-1);
	    if(maxLog > 1) dc3 += a3k3[i] *mell_L(i,3,N-1);
	    if(maxLog > 2) dc3 += a3k2[i] *mell_L(i,2,N-1);
	    if(maxLog > 3) dc3 += a3k1[i] *mell_L(i,1,N-1);
	    if(maxLog > 4) dc3 += a3k0[i] *mell_L(i,0,N-1);
	  }
	}
      }
      if(APtype==2 && fullNS) {
	dc1 = 0; // reset
	dc2 = 0; // reset
	dc3 = 0; // reset
	//
	// full NS in z-space
	dc3 = 189.*mell_L(0,4,N) -110.333*mell_L(0,3,N) - 1236.77*mell_L(0,2,N) + 2560.34*mell_L(0,1,N) -1375.07*mell_L(0,0,N);
	//
	if(NTSexpansion == 0) {   // N-space with psi0(N)
	  L = psi0(N);
	  double c3k5[] = {0, 0, 0, -1296., 2592., -6480., 12960., -27216., 54432., -110160.};
	  double c3k4[] = {0, -61.0586, 754.059, -4238.27, 5803.68, -2831.34, -4045.14, 41105.7, -124248., 356251.};
	  double c3k3[] = {0, -599.372, 63.8296, -10150.5, 6991.69, -16682.6, 21726.9, -57884.1, 82044.2, -196390.};
	  for(int i=1; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += L*L*L*L*L*c3k5[i] /pow(N,i+1);
	    if(maxLog > 0) dc3 += L*L*L*L  *c3k4[i] /pow(N,i+1);
	    if(maxLog > 1) dc3 += L*L*L    *c3k3[i] /pow(N,i+1);
	  }
	}
	if(NTSexpansion == 1) {   // N-space with psi0(N+1)
	  L = psi0(N+1);
	  double c3k5[] = {0, 0, 0, -1296., 2592., -6480., 12960., -27216., 54432., -110160.};
	  double c3k4[] = {0, -61.0586, 754.059, -4238.27, 12283.7, -15791.3, 28354.9, -23694.3, 11832.3, 84091.};
	  double c3k3[] = {0, -599.374, 308.064, -13166.7, 23944.8, -52857.3, 58972.3, -106504., 47221.5, 28441.3};
	  for(int i=1; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += L*L*L*L*L*c3k5[i] /pow(N,i+1);
	    if(maxLog > 0) dc3 += L*L*L*L  *c3k4[i] /pow(N,i+1);
	    if(maxLog > 1) dc3 += L*L*L    *c3k3[i] /pow(N,i+1);
	  }
	}
	if(NTSexpansion == 2) {   // N-space with psi0(N+1) and 1/N/(N+1)/(N+2)...
	  L = psi0(N+1);
	  double c3k5[] = {0, 0, 0, -1296., -5184., -25920., -155520., -1.08864e6, -8.70912e6, -7.83821e7};
	  double c3k4[] = {0, -61.0586, 693., -2098.21, -5217.64, -5056.42, 81272.1, 1.24003e6, 1.45652e7, 1.67715e8};
	  double c3k3[] = {0, -599.374, -291.31, -13441.3, -55263.2, -273227., -1.64861e6, -1.17896e7, -9.73648e7, -9.10135e8};
	  for(int i=1; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += L*L*L*L*L*c3k5[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 0) dc3 += L*L*L*L  *c3k4[i] *lGamma(N)/lGamma(N+1+i);
	    if(maxLog > 1) dc3 += L*L*L    *c3k3[i] *lGamma(N)/lGamma(N+1+i);
	  }
	}
	if(NTSexpansion == 3) {   // z-space
	  double a3k5[] = {0, 0, 0, 216., 216., 216., 216., 216., 216., 216.};
	  double a3k4[] = {0, -61.0586, 346.5, -1706.31, -1034.01, -696.744, -469.729, -297.999, -159.653, -43.714};
	  double a3k3[] = {0, 702.633, -1255.44, 7675.63, 2971.25, 1516., 830.977, 454.002, 232.367, 100.178};
	  for(int i=1; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += a3k5[i] *mell_L(i,5,N);
	    if(maxLog > 0) dc3 += a3k4[i] *mell_L(i,4,N);
	    if(maxLog > 1) dc3 += a3k3[i] *mell_L(i,3,N);
	  }
	}
	if(NTSexpansion == 4) {   // z-space, divided by z
	  double a3k5[] = {0, 0, 0, 216., 0, 0, 0, 0, 0, 0};
	  double a3k4[] = {0, -61.0586, 407.559, -2052.81, 672.3, 337.265, 227.015, 171.73, 138.346, 115.939};
	  double a3k3[] = {0, 702.632, -1958.07, 8931.08, -4704.38, -1455.25, -685.021, -376.975, -221.635, -132.189};
	  for(int i=1; i<maxorder; i++) {
	    if(maxLog >-1) dc3 += a3k5[i] *mell_L(i,5,N-1);
	    if(maxLog > 0) dc3 += a3k4[i] *mell_L(i,4,N-1);
	    if(maxLog > 1) dc3 += a3k3[i] *mell_L(i,3,N-1);
	  }
	}
      }
    } else {
      //cout << "Next-to-soft not implemented for this choice of logs" << endl;
    }
    if(order == 1) res += dc1;
    if(order == 2) res += dc2 + W1*dc1;
    if(order == 3) res += dc3 + W1*dc2 + W2*dc1;
  }
  if(dFMMV) {
    if(__log_type__==3 && APtype==0) {
      dcomplex L = log(N);
      dcomplex c1 =   6.*(L + EulerGamma)/N;
      dcomplex c2 =  36.*(L*L*L + 2.926*L*L + 5.970*L + 2.007)/N;
      dcomplex c3 = 108.*(L*L*L*L*L + 5.701*L*L*L*L + 18.9*L*L*L + 46.*L*L + 18.*L + 9.)/N;
      double W1 = (finite_mt ? _W1t : _W1);
      double W2 = (finite_mt ? _W2t : _W2);
      if(order == 1) res += c1;
      if(order == 2) res += c2 + W1*c1;
      if(order == 3) res += c3 + W1*c2 + W2*c1;
    } else {
      cout << "dFMMV not implemented for our result" << endl;
      //if(order == 2) res += pow(CA/4.,2)* 56*mell_L(0,2,N);
      //if(order == 3) res += pow(CA/4.,3)*448*mell_L(0,4,N);
    }
  }
  return res;
}
#include "fixed-order_Nspace.cc"
dcomplex FO_3_qg(dcomplex N) {
  dcomplex FO3qg = 0;
  if(NTSexpansion == 0) {   // N-space with psi0(N)
    dcomplex L = psi0(N);
    FO3qg = (-121.681 - 556.332*L - 282.17*L*L - 242.565*L*L*L - 53.4403*L*L*L*L - 17.8194*L*L*L*L*L) /N;
  }
  if(NTSexpansion == 1 || NTSexpansion == 2) {   // N-space with psi0(N+1)
    dcomplex L = psi0(N+1);
    FO3qg = (-121.681 - 556.332*L - 282.17*L*L - 242.565*L*L*L - 53.4403*L*L*L*L - 17.8194*L*L*L*L*L) /N;
  }
  if(NTSexpansion == 3) {   // z-space
    FO3qg = 1283./72.*mell_L(0,5,N) - 5215./2592.*mell_L(0,4,N) -114.569021*mell_L(0,3,N) +513.56298*mell_L(0,2,N) -313.98523*mell_L(0,1,N) +204.62079*mell_L(0,0,N);
  }
  if(NTSexpansion == 4) {   // z-space, divided by z
    FO3qg = 1283./72.*mell_L(0,5,N-1) - 5215./2592.*mell_L(0,4,N-1) -114.569021*mell_L(0,3,N-1) +513.56298*mell_L(0,2,N-1) -313.98523*mell_L(0,1,N-1) +204.62079*mell_L(0,0,N-1);
  }
  //
  //double W1 = (finite_mt ? _W1t : _W1);
  //double W2 = (finite_mt ? _W2t : _W2);
  return FO3qg;// + W1*NFO_2_qg(N,muM) + (W2-W1*W1)*NFO_1_qg(N,muM);
}


const double MPbeta = 1;
const unsigned int MPgamma = 0;
const unsigned int Nchebyshev = 13;
double ChebCoeff_gg[Nchebyshev+1];
double ChebCoeff_qg[Nchebyshev+1];
double ChebCoeff_SS[Nchebyshev+1];
double ChebCoeff_TT[Nchebyshev+1];
double ChebCoeff_VV[Nchebyshev+1];
dcomplex LuminosityN(dcomplex N, double *ChebCoeff) {
  dcomplex value = 0;
  for(unsigned int i=0; i<=Nchebyshev; i++) {
    for(unsigned int j=0; j<=MPgamma; j++) {
      value += ChebCoeff[i] * pow(-1.,j) * gsl_sf_choose(MPgamma,j) / pow(N+j-MPbeta,i+1.);
    }
  }
  return value;
}
dcomplex lum_times_g0expS(dcomplex N) {
  dcomplex res = g0expS(N) * LuminosityN(N, ChebCoeff_gg);
  if(order == 3 && NextToSoft && __all_channels__) res += FO_3_qg(N) * LuminosityN(N, ChebCoeff_qg);
  return res;
}

// exact muF dependence
dcomplex muFdep_1(dcomplex N) {
  double lf = -2.*log(_muF_mu_ratio);
  //
  dcomplex g0gg = M_PI*gamma0gg(N,_nf);
  dcomplex g0gq = M_PI*gamma0gq(N);
  //
  // full
  dcomplex FO1gg = 0;
  FO1gg += 2*(g0gg-b0)*lf;
  //
  dcomplex FO1qg = 0;
  FO1qg += g0gq*lf;
  //
  dcomplex res = 0;
  if(_gg_channel_) res += FO1gg * LuminosityN(N, ChebCoeff_gg);
  if(__all_channels__) {
    if(_qg_channel_) res += FO1qg * LuminosityN(N, ChebCoeff_qg);
  }
  return res;
}
dcomplex muFdep_2(dcomplex N) {
  double mH = muM;
  double lf = -2.*log(_muF_mu_ratio);
  //
  dcomplex C1qg = NFO_1_qg(N,mH);
  //
  //dcomplex C1gg = g0expS_order1(N);
  dcomplex C1gg = NFO_1_gg(N,mH);
  //
  dcomplex C1qqb = NFO_1_qqb(N,mH);
  // singlet
  dcomplex C1SS = C1qqb/(2.*_nf);
  // non-singlet
  dcomplex C1TT = C1qqb/2.;
  dcomplex C1VV = -C1qqb/2.;
  //
  dcomplex g0gg = M_PI*gamma0gg(N,_nf);
  dcomplex g0gq = M_PI*gamma0gq(N);
  dcomplex g0qg = M_PI*gamma0qg(N,_nf);
  dcomplex g0qq = M_PI*gamma0qq(N);
  //
  gamma1sums g1s;
  sums(N,g1s);
  dcomplex g1gg = M_PI*M_PI*gamma1SGgg(N,_nf,g1s);
  dcomplex g1gq = M_PI*M_PI*gamma1SGgq(N,_nf,g1s);
  //
  // full
  dcomplex FO2gg = 0;
  FO2gg += (C1gg*(6*b0 - 4*g0gg) +4*b1 - 4*C1qg*g0qg -4*g1gg)*(-lf/2.)
    + (3*pow(b0,2) - 5*b0*g0gg + 2*pow(g0gg,2) + g0gq*g0qg)*pow(-lf,2);  // with muR = muF
  //FO2gg = pow(lf,2)*(b0*g0gg + 2*pow(g0gg,2) + g0gq*g0qg) + 2*lf*(C1gg*g0gg + C1qg*g0qg + g1gg);  // with muR = mH
  //
  dcomplex FO2qg = 0;
  FO2qg += (C1gg*g0gq + C1SS*g0qg + C1qg*(-3*b0 + g0gg + g0qq) + g1gq)*lf
    + g0gq*(-5*b0 + 3*g0gg + g0qq)*pow(lf,2)/2.;
  //
  dcomplex FO2SS = 0;
  FO2SS += (C1SS*(3*b0 - 2*g0qq) - 2*C1qg*g0gq)*(-lf) + g0gq*g0gq*lf*lf;
  //
  dcomplex FO2TT = 0;
  FO2TT += C1TT*(3*b0 - 2*g0qq)*(-lf);
  //
  dcomplex FO2VV = 0;
  FO2VV += C1VV*(3*b0 - 2*g0qq)*(-lf);
  //
  dcomplex res = 0;
  if(_gg_channel_) res += FO2gg * LuminosityN(N, ChebCoeff_gg);
  if(__all_channels__) {
    if(_qg_channel_) res += FO2qg * LuminosityN(N, ChebCoeff_qg);
    if(_qq_channel_) res += FO2SS * LuminosityN(N, ChebCoeff_SS) + FO2TT * LuminosityN(N, ChebCoeff_TT) + FO2VV * LuminosityN(N, ChebCoeff_VV);
  }
  return res;
}
dcomplex muFdep_3(dcomplex N) {
  double mu = muM;
  double lf = -2.*log(_muF_mu_ratio);
  // scale dependence
  //dcomplex C1qg = CF*mell_L(0,1,N) + CF/2./N;
  //double xht = pow(mu/mtop,2);
  //dcomplex C2qg = 367./54.*mell_L(0,3,N) + (85./36.+_nf/18.)*mell_L(0,2,N) + (341./18.-50./9.*ZETA2-_nf*2./3.)*mell_L(0,1,N) + (11./27.+29./6.*ZETA2+311./18.*ZETA3+_nf*13./81.)
  //  + xht*(136./405.*mell_L(0,1,N) + 68./405.)
  //  + xht*xht*(3553./85050.*mell_L(0,1,N) + 3553./170100.);
  dcomplex C1qg = NFO_1_qg(N,mu);
  dcomplex C2qg = NFO_2_qg(N,mu);
  //
  dcomplex C1gg = NFO_1_gg(N,mu);
  dcomplex C2gg = NFO_2_gg(N,mu);
  //
  dcomplex C1qqb = NFO_1_qqb(N,mu);
  dcomplex C2qq  = NFO_2_qq (N,mu);
  dcomplex C2qqb = NFO_2_qqb(N,mu);
  dcomplex C2qqp = NFO_2_qqp(N,mu);
  //cout << "C1gg = " << C1gg << endl;
  //cout << "C1qg = " << C1qg << endl;
  //cout << "C1qqb= " << C1qqb << endl<< endl;
  //cout << "C2gg = " << C2gg << endl;
  //cout << "C2qg = " << C2qg << endl;
  //cout << "C2qqb= " << C2qqb << endl;
  //cout << "C2qq = " << C2qq  << endl;
  //cout << "C2qqp= " << C2qqp << endl<< endl;
  // singlet
  dcomplex C1SS = C1qqb/(2.*_nf);
  dcomplex C2SS = (C2qq + C2qqb + 2*(_nf-1.)*C2qqp)/(2.*_nf);
  // non-singlet
  dcomplex C1TT = C1qqb/2.;
  dcomplex C2TT = (C2qq + C2qqb - 2*C2qqp)/2.;
  dcomplex C1VV = -C1qqb/2.;
  dcomplex C2VV = (C2qq - C2qqb)/2.;
  //
  dcomplex g0gg = M_PI*gamma0gg(N,_nf);
  dcomplex g0gq = M_PI*gamma0gq(N);
  dcomplex g0qg = M_PI*gamma0qg(N,_nf);
  dcomplex g0qq = M_PI*gamma0qq(N);
  //cout << "g0gg = " << g0gg << endl;
  //cout << "g0qg = " << g0qg << endl;
  //cout << "g0gq = " << g0gq << endl;
  //cout << "g0qq = " << g0qq << endl<< endl;
  //
  gamma1sums g1s;
  sums(N,g1s);
  dcomplex g1gg = M_PI*M_PI*gamma1SGgg(N,_nf,g1s);
  dcomplex g1gq = M_PI*M_PI*gamma1SGgq(N,_nf,g1s);
  dcomplex g1qg = M_PI*M_PI*gamma1SGqg(N,_nf,g1s);
  dcomplex g1qq = M_PI*M_PI*gamma1SGqq(N,_nf,g1s); // this is the singlet = NSplus + PS 
  dcomplex g1mn = M_PI*M_PI*gamma1NSminus(N,_nf,g1s);
  dcomplex g1pl = M_PI*M_PI*gamma1NSplus(N,_nf,g1s);
  //cout << "g1gg = " << g1gg << endl;
  //cout << "g1qg = " << g1qg << endl;
  //cout << "g1gq = " << g1gq << endl;
  //cout << "g1qq = " << g1qq << endl;
  //cout << "g1mn = " << g1mn << endl;
  //cout << "g1pl = " << g1pl << endl<< endl;
  //
  dcomplex g2gg, g2gq, g2qg, g2ps;
  p2ijp(g2ps, g2qg, g2gq, g2gg, N, _nf);
  g2gg /= 64.;
  g2gq /= 64.;
  g2qg /= 64.;
  g2ps /= 64.;
  //g2gq *= 20;
  //dcomplex g2pl, g2mn, g2vl;
  //p2nsp(g2pl, g2mn, g2vl, N, _nf);
  //g2pl /= 64.;
  //g2mn /= 64.;
  //g2vl /= 64.;
  //cout << "g2gg = " << g2gg << endl;
  //cout << "g2qg = " << g2qg << endl;
  //cout << "g2gq = " << g2gq << endl;
  //cout << "g2ps = " << g2ps << endl<< endl;
  //
  // full
  dcomplex FO3gg = 0;
  if(newevolmuFmuR)
    FO3gg = pow(lf,3)*((2*pow(b0,2)*g0gg)/3. + b0*(2*pow(g0gg,2) + g0gq*g0qg) + (4*pow(g0gg,3) + 5*g0gg*g0gq*g0qg + g0gq*g0qg*g0qq)/3.) + 
      pow(lf,2)*(C1gg*b0*g0gg + b1*g0gg + C1qg*b0*g0qg + C1SS*pow(g0qg,2) + C1gg*(2*pow(g0gg,2) + g0gq*g0qg) + C1qg*g0qg*(3*g0gg + g0qq) + 2*b0*g1gg + 4*g0gg*g1gg + 
		 g0qg*g1gq + g0gq*g1qg)
      + 2*lf*(C2gg*g0gg + C2qg*g0qg + C1gg*g1gg + C1qg*g1qg + g2gg);  // with muR=mH
  else
    FO3gg = (C2gg*(8*b0 - 4*g0gg) + C1gg*(6*b1 - 4*g1gg) - 4*(-b2 + C2qg*g0qg + C1qg*g1qg + g2gg))*(-lf/2.)
      + (-5*b1*g0gg + 3*C1qg*g0gg*g0qg + C1SS*pow(g0qg,2) + C1gg*(6*pow(b0,2) - 7*b0*g0gg + 2*pow(g0gg,2) + g0gq*g0qg) + C1qg*g0qg*g0qq + 
	 b0*(7*b1 - 7*C1qg*g0qg - 6*g1gg) + 4*g0gg*g1gg + g0qg*g1gq + g0gq*g1qg)*pow(lf,2)
      + (8*(12*pow(b0,3) - 26*pow(b0,2)*g0gg - 4*pow(g0gg,3) - 5*g0gg*g0gq*g0qg + 9*b0*(2*pow(g0gg,2) + g0gq*g0qg) - g0gq*g0qg*g0qq)*pow(-lf/2.,3))/3.;  // with muR=muF
  //
  dcomplex FO3qg = 0;
  if(newevolmuFmuR)
    FO3qg = (pow(lf,3)*(pow(b0,2)*g0gq + (3*b0*g0gq*(3*g0gg + g0qq))/2. + (g0gq*(7*pow(g0gg,2) + 4*g0gq*g0qg + 4*g0gg*g0qq + pow(g0qq,2)))/2.))/3. + 
      (pow(lf,2)*((3*C1gg*b0*g0gq)/2. + (3*b1*g0gq)/2. + (3*C1SS*b0*g0qg)/2. + (3*C1qg*b0*(g0gg + g0qq))/2. + 3*b0*g1gq + 
		  (3*(C1SS*g0gg*g0qg + 3*C1SS*g0qg*g0qq + C1gg*g0gq*(3*g0gg + g0qq) + C1qg*(pow(g0gg,2) + 4*g0gq*g0qg + 2*g0gg*g0qq + pow(g0qq,2)) + 3*g0gq*g1gg + 
		      3*g0gg*g1gq + g0qq*g1gq + g0gq*g1qq))/2.))/3. + lf*(C2gg*g0gq + C2SS*g0qg + C2qg*(g0gg + g0qq) + C1qg*g1gg + C1gg*g1gq + C1SS*g1qg + C1qg*g1qq + g2gq);
  else
    FO3qg = 2*(C2gg*g0gq + C2SS*g0qg + C2qg*(-4*b0 + g0gg + g0qq) + C1gg*g1gq + C1SS*g1qg + C1qg*(-3*b1 + g1gg + g1qq) + g2gq)*lf/2.
      + 2*(-5*b1*g0gq - 7*C1SS*b0*g0qg + C1SS*g0gg*g0qg + 3*C1SS*g0qg*g0qq + C1gg*g0gq*(-7*b0 + 3*g0gg + g0qq)
	   + C1qg*(12*pow(b0,2) + pow(g0gg,2) + 4*g0gq*g0qg + 2*g0gg*g0qq + pow(g0qq,2) - 7*b0*(g0gg + g0qq))
	   + 3*g0gq*g1gg - 6*b0*g1gq + 3*g0gg*g1gq + g0qq*g1gq + g0gq*g1qq)*pow(lf/2.,2)
      + 4*g0gq*(26*pow(b0,2) + 7*pow(g0gg,2) + 4*g0gq*g0qg + 4*g0gg*g0qq + pow(g0qq,2) - 9*b0*(3*g0gg + g0qq))*pow(lf/2.,3)/3.;
  //
  dcomplex FO3SS = 0;
  if(newevolmuFmuR)
    FO3SS = pow(lf,3)*(b0*pow(g0gq,2) + pow(g0gq,2)*(g0gg + g0qq)) + 
      pow(lf,2)*(C1qg*b0*g0gq + C1gg*pow(g0gq,2) + C1SS*g0gq*g0qg + C1SS*b0*g0qq + 2*C1SS*pow(g0qq,2) + C1qg*g0gq*(g0gg + 3*g0qq) + 2*g0gq*g1gq) + 
      2*lf*(C2qg*g0gq + C2SS*g0qq + C1qg*g1gq + C1SS*g1qq);
  else
    FO3SS = (8*C2SS*b0 + 6*C1SS*b1 - 4*C2qg*g0gq - 4*C2SS*g0qq - 4*C1qg*g1gq - 4*C1SS*g1qq)*(-lf/2.)
      + 4*(C1SS*(6*pow(b0,2) + g0gq*g0qg - 7*b0*g0qq + 2*pow(g0qq,2)) + g0gq*(C1gg*g0gq + C1qg*(-7*b0 + g0gg + 3*g0qq) + 2*g1gq))*pow(lf/2.,2)
      + 8*pow(g0gq,2)*(3*b0 - g0gg - g0qq)*pow(-lf/2.,3);
  //
  dcomplex FO3TT = 0;
  if(newevolmuFmuR)
    FO3TT = lf*(2*C2TT*g0qq + C1TT*(lf*g0qq*(b0 + 2*g0qq) + 2*g1pl));
  else
    FO3TT += (4*C2TT*b0 + 3*C1TT*b1 - 2*C2TT*g0qq - 2*C1TT*g1pl)*(-lf)
      + (C1TT*(6*pow(b0,2) - 7*b0*g0qq + 2*pow(g0qq,2)))*pow(lf,2);
  //
  dcomplex FO3VV = 0;
  if(newevolmuFmuR)
    FO3VV = lf*(2*C2VV*g0qq + C1VV*(lf*g0qq*(b0 + 2*g0qq) + 2*g1mn));
  else
    FO3VV += (4*C2VV*b0 + 3*C1VV*b1 - 2*C2VV*g0qq - 2*C1VV*g1mn)*(-lf)
      + (C1VV*(6*pow(b0,2) - 7*b0*g0qq + 2*pow(g0qq,2)))*pow(lf,2);
  //
  dcomplex res = 0;
  if(_gg_channel_) res += FO3gg * LuminosityN(N, ChebCoeff_gg);
  if(__all_channels__) {
    if(_qg_channel_) res += FO3qg * LuminosityN(N, ChebCoeff_qg);
    if(_qq_channel_) res += FO3SS * LuminosityN(N, ChebCoeff_SS) + FO3TT * LuminosityN(N, ChebCoeff_TT) + FO3VV * LuminosityN(N, ChebCoeff_VV);
  }
  return res;
}
dcomplex muFdep(dcomplex N) {
  if(order==1) return muFdep_1(N);
  if(order==2) return muFdep_2(N);
  if(order==3) return muFdep_3(N);
  return 0;
}
double MellinInversionIntegrand(double t, void *p) {
  double x = *(double*)p;
  dcomplex N = MPbeta + 3. + (1.-I)*log(t);
  dcomplex res = 0;
  if(computesoft) res += lum_times_g0expS(N);
  if(exactmuFdep) res += muFdep(N);
  res *= (I-1.) * exp(-N*log(x)) /t;
  return imag(res)/M_PI;
}
double MellinInversion(double tau, double *err=NULL) {
  double res=0, prec = 1.e-4;
  //gsl_error_handler_t *old_handler = gsl_set_error_handler (NULL);
  //gsl_set_error_handler_off();
  res = integration::gauss(MellinInversionIntegrand, 0., 1., prec, err, &tau);
  //gsl_set_error_handler (old_handler);
  double lgg = Lum(tau, muFpdf);
  *err /= lgg;
  return res/lgg;
}

